<?php
/**
 * Plugin Name:     Holler Box FOMO Notifications
 * Plugin URI:      http://hollerwp.com
 * Description:     Show FOMO notifications to visitors, integrates with a variety of plugins such as WooCommerce, EDD, GiveWP, and more.
 * Version:         0.4.1
 * Author:          Scott Bolinger
 * Author URI:      http://hollerwp.com
 * Text Domain:     hollerbox-sales
 *
 * @author          Scott Bolinger
 * @copyright       Copyright (c) Scott Bolinger 2017
 *
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'Holler_Box_Sales' ) ) {

    /**
     * Main Holler_Box_Sales class
     *
     * @since       0.1.0
     */
    class Holler_Box_Sales {

        /**
         * @var         Holler_Box_Sales $instance The one true Holler_Box_Sales
         * @since       0.1.0
         */
        private static $instance;


        /**
         * Get active instance
         *
         * @access      public
         * @since       0.1.0
         * @return      object self::$instance The one true Holler_Box_Sales
         */
        public static function instance() {
            if( !self::$instance ) {
                self::$instance = new Holler_Box_Sales();
                self::$instance->setup_constants();
                self::$instance->includes();
                self::$instance->load_textdomain();
                self::$instance->hooks();
            }

            return self::$instance;
        }


        /**
         * Setup plugin constants
         *
         * @access      private
         * @since       0.1.0
         * @return      void
         */
        private function setup_constants() {
            // Plugin version
            define( 'Holler_Box_Sales_VER', '0.4.1' );

            // Plugin path
            define( 'Holler_Box_Sales_DIR', plugin_dir_path( __FILE__ ) );

            // Plugin URL
            define( 'Holler_Box_Sales_URL', plugin_dir_url( __FILE__ ) );

            // this is the URL our updater / license checker pings. This should be the URL of the site with EDD installed
            define( 'HWP_SALES_STORE_URL', 'https://hollerwp.com' );
            define( 'HWP_SALES_ID', 156 );
            define('Holler_Box_Sales_File', __FILE__ );
        }


        /**
         * Include necessary files
         *
         * @access      private
         * @since       0.1.0
         * @return      void
         */
        private function includes() {

            require_once Holler_Box_Sales_DIR . 'includes/class-holler-sales-functions.php';

            if( is_admin() )
                require_once Holler_Box_Sales_DIR . 'includes/class-holler-sales-admin.php';

            if( is_admin() && !class_exists( 'EDD_SL_Plugin_Updater' ) )
                include( dirname( __FILE__ ) . '/includes/EDD_SL_Plugin_Updater.php' );
            
        }


        /**
         * Run action and filter hooks
         *
         * @access      private
         * @since       0.1.0
         * @return      void
         *
         *
         */
        private function hooks() {

        }


        /**
         * Internationalization
         *
         * @access      public
         * @since       0.1.0
         * @return      void
         */
        public function load_textdomain() {

            load_plugin_textdomain( 'hollerbox-sales' );
            
        }

        /**
         * Holler Box Free Required admin notice
         *
         * @access      public
         * @since       0.1.0
         * @return      void
         */
        public static function hwp_required() {
            echo '<div id="message" class="error"><p>'. sprintf( __( '%1$s requires Holler Box and Holler Box Pro to be installed/activated. Please install under Plugins => Add New. %1$s has been deactivated.', 'hollerbox' ), '<b>Holler Box Sales Notifications</b> ' ) .'</p></div>';
            deactivate_plugins( plugin_basename( __FILE__ ), true );
        }

    }
} // End if class_exists check


/**
 * The main function responsible for returning the one true EDD_Metrics
 * instance to functions everywhere
 *
 * @since       0.1.0
 * @return      Holler_Box_Sales::instance()
 *
 */
function holler_box_sales_load() {

    // is main plugin active? If not, throw a notice and deactivate
    if ( ! in_array( 'holler-box/holler-box.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
        add_action( 'all_admin_notices', array( 'Holler_Box_Sales', 'hwp_required' ) );
        return;
    }

    return Holler_Box_Sales::instance();
}
add_action( 'plugins_loaded', 'holler_box_sales_load' );


/**
 * The activation hook is called outside of the singleton because WordPress doesn't
 * register the call from within the class, since we are preferring the plugins_loaded
 * hook for compatibility, we also can't reference a function inside the plugin class
 * for the activation function. If you need an activation function, put it here.
 *
 * @since       0.1.0
 * @return      void
 */
function holler_box_sales_activation() {
    /* Activation functions here */
}
register_activation_hook( __FILE__, 'holler_box_sales_activation' );