<?php
/**
 * Holler Box Pro Functions
 * @since       0.1.0
 */


// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

if( !class_exists( 'Holler_Sales_Functions' ) ) {

    /**
     * Holler_Sales_Functions class
     *
     * @since       0.2.0
     */
    class Holler_Sales_Functions {

        /**
         * @var         Holler_Sales_Functions $instance The one true Holler_Sales_Functions
         * @since       0.2.0
         */
        private static $instance;
        public static $errorpath = '../php-error-log.php';
        public static $active = array();
        // sample: error_log("meta: " . $meta . "\r\n",3,self::$errorpath);

        /**
         * Get active instance
         *
         * @access      public
         * @since       0.2.0
         * @return      object self::$instance The one true Holler_Sales_Functions
         */
        public static function instance() {
            if( !self::$instance ) {
                self::$instance = new Holler_Sales_Functions();
                self::$instance->hooks();
            }

            return self::$instance;
        }


        /**
         * Include necessary files
         *
         * @access      private
         * @since       0.2.0
         * @return      void
         */
        private function hooks() {

            add_action( 'wp_enqueue_scripts', array( $this, 'scripts_styles' ) );

            add_action( 'wp_ajax_nopriv_hwp_fomo_ajax', array( $this, 'fomo_ajax' ) );
            add_action( 'wp_ajax_hwp_fomo_ajax', array( $this, 'fomo_ajax' ) );

        }

        /**
         * Load scripts
         *
         * @since       0.4.0
         * @return      void
         */
        public function scripts_styles( $hook ) {

            // Use minified libraries if SCRIPT_DEBUG is turned off
            $suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

            //wp_enqueue_script( 'holler-js', Holler_Box_Sales_URL . 'assets/js/holler-frontend' . $suffix . '.js', array( 'jquery' ), Holler_Box_Sales_VER, true );
            wp_enqueue_style( 'holler-fomo-css', Holler_Box_Sales_URL . 'assets/css/holler-box-fomo' . $suffix . '.css', null, Holler_Box_Sales_VER );

        }

        /**
         * Get sale with ajax
         *
         * @since       0.2.0
         * @return      void
         */
        public function fomo_ajax() {

            $int = ( isset( $_GET['int'] ) ? $_GET['int'] : 1 );
            $id = $_GET['id'];

            if( !$id )
                wp_send_json_error('Error: No ID detected.');

            $integration = get_post_meta( $id, 'fomo_integration', 1 );

            // detect ecomm plugin
            if( $integration === 'edd' ) {

                $content = $this->edd_sale( $int, $id );

            } elseif( $integration === 'woo' ) {
                
                $content = $this->woocommerce_sale( $int, $id );

            } elseif( $integration === 'give' ) {
                
                $content = $this->give_query( $int, $id );

            } elseif( $integration === 'memberpress' ) {
                
                $content = $this->memberpress_sale( $int, $id );

            } elseif( $integration === 'pmp' ) {
                
                $content = $this->pmp_sale( $int, $id );

            } elseif( empty( $integration ) || $integration === '' ) {

                // for backwards compatibility before integration setting was added
                if( class_exists('Easy_Digital_Downloads') ) {

                    $content = $this->edd_sale( $int, $id );

                } elseif( class_exists('WooCommerce') ) {
                    
                    $content = $this->woocommerce_sale( $int, $id );

                }

            }

            wp_send_json_success( $content );
    
        } 

        /**
         * Add latest EDD sale to Hollerbox
         *
         * @since       0.1.0
         * @return      void
         */
        public function edd_sale( $int, $box_id ) {

            // handle less than 10 products
            $total = $this->count_posts( 'edd_payment' );
            if( $total < 10 )
                $int = rand( 0, intval( $total ) - 1 );

            // If past 10 posts, get a random one
            if( $int > 10 )
                $int = rand( 1, 10 );

            $edd_payment = get_post_type_object( 'edd_payment' );
            $args = array(
                'post_type' => 'edd_payment',
                'post_status' => array( 'publish' ),
                'posts_per_page' => 1,
                'offset' => $int,
            );

            $return = self::edd_payment_loop( $args, $box_id );

            return $return;

        }

        /**
         * Loop through EDD payments to get a FOMO sale
         *
         * @since       0.3.0
         * @return      void
         */
        public function edd_payment_loop( $args, $box_id ) {

            $content = apply_filters( 'hwp_fomo_pre_edd_payment_loop_content', false, $args );

            if ( $content ) {
                return $content;
            }

            $fomo_image = get_post_meta( $box_id, 'fomo_image', 1 );

            // The Query
            $the_query = new WP_Query( $args );

            if ( $the_query->have_posts() ) {
                while ( $the_query->have_posts() ) {
                    $the_query->the_post();

                    $post_id = get_the_ID();
                    $download_meta = get_post_meta( $post_id, '_edd_payment_meta', 1 );
                    $download_id = $download_meta['downloads'][0]['id'];
                    

                    // If using product image setting, show that instead of avatar
                    if( $fomo_image === 'product') {

                        $download_thumb_url = get_the_post_thumbnail_url( $download_id, 'thumbnail' );
                        $content .= '<div class="hwp-product-image" style="background-image: url(' . $download_thumb_url . ');"></div>';

                    } else {

                        $customer_email = $download_meta['user_info']['email'];
                        $content .= get_avatar( $customer_email, 50 );

                    }

                    $download = edd_get_download( $download_id );
                    $edd_link = get_permalink( $download_id );

                    $content .= '<div class="fomo-text ' . $fomo_image . '">';

                    $content .= '<span class="fomo-name">' . $download_meta['user_info']['first_name'] . ' purchased </span>';
                    $content .= '<span class="fomo-product"><a href="' . apply_filters( 'hwp_fomo_edd_link', $edd_link, $download_id ) . '">' . $download->post_title . '</a></span>';

                    $content .= '</div>';

                }
                wp_reset_postdata();
            }
            //return serialize( $download );
            return apply_filters( 'hwp_fomo_content', $content, $post_id, $box_id );
        }

        /**
         * Add latest Woo sale to Hollerbox
         *
         * @since       0.1.0
         * @return      void
         */
        public function woocommerce_sale( $int, $box_id ) {

            $fomo_image = get_post_meta( $box_id, 'fomo_image', 1 );

            // handle less than 10 products
            $total = $this->count_posts( 'shop_order' );
            if( $total < 10 )
                $int = rand( 0, intval( $total ) - 1 );

            // If past 10 posts, get a random one
            if( $int > 10 )
                $int = rand( 1, 10 );

            $args = array( 
                'status' => 'wc-completed',
                'type' => 'shop_order',
                'posts_per_page' => 1,
                'offset'   => $int
             );

            $order = wc_get_orders( $args )[0];
            $id = $order->get_id();

            $items = $order->get_items();
            $content;

            foreach ( $items as $item ) {
                $name = $item['name'];
                $product_id = $item['product_id'];
            }

            // If using product image setting, show that instead of avatar
            if( $fomo_image === 'product') {

                $product_image = get_the_post_thumbnail_url( $product_id, 'thumbnail' );
                $content .= '<div class="hwp-product-image" style="background-image: url(' . $product_image . ');"></div>';

            } else {

                $email = $order->get_billing_email();

                $avatar_url = get_avatar_url( $email, array( 'size' => 50 ) );
                $content .= '<img alt="" src="' . $avatar_url . '" class="avatar avatar-50 photo" height="50" width="50" />';

            }

            $first_name = $order->get_billing_first_name();
            
            $content .= '<div class="fomo-text ' . $fomo_image . '">';
            $content .= '<span class="fomo-name">' . $first_name . ' purchased </span>';
            $content .= '<span class="fomo-product"><a href="' . get_permalink( $product_id ) . '">' . $name . '</a></span>';
            $content .= '</div>';

            return apply_filters( 'hwp_fomo_content', $content, $product_id, $box_id );

        }

        /**
         *  Show GiveWP Donation Information
         *
         *  This query outputs the latest 3 transactions
         *  with the name of the donor and the amount
         *  https://github.com/WordImpress/Give-Snippet-Library/blob/master/useful-queries/sample-transaction-queries.php
         */
        public function give_query( $int, $box_id ) {

            $fomo_image = get_post_meta( $box_id, 'fomo_image', 1 );

            $total = $this->count_posts( 'give_payment' );

            if( $total < 10 )
                $int = rand( 0, intval( $total ) - 1 );

            // If past 10 posts, get a random one
            if( $int > 10 )
                $int = rand( 1, 10 );

            // Query Arguments
            $args3 = array(
                'post_type'      => 'give_payment',
                'posts_per_page' => 1,
                'offset' => $int,
            );

            $loop3 = new WP_Query( $args3 );

            if ( $loop3->have_posts() ) :

                /** 
                 * Get donation info
                 */
                while ( $loop3->have_posts() ) : $loop3->the_post();

                    $post_id = get_the_ID();
                    $meta = get_post_meta( $post_id );
                    // Transaction have their own metadata; let's get it.
                    $paymentmeta = $meta['_give_payment_meta'];
                    // The metadata is serialized. Let's pull that apart.
                    $getmeta = maybe_unserialize( $paymentmeta[0] );
                    // Now that we've got it, we can define the name
                    $firstname = $getmeta['user_info']['first_name'];
                    $email  = $getmeta['user_info']['email'];
                    $total = intval( $meta['_give_payment_total'][0] );
                    $formtitle = get_the_title( $getmeta['form_id'] );

                    // If using product image setting, show that instead of avatar
                    if( $fomo_image === 'product') {

                        $form_thumb = get_the_post_thumbnail_url( $getmeta['form_id'], 'thumbnail' );
                        $content .= '<div class="hwp-product-image" style="background-image: url(' . $form_thumb . ');"></div>';

                    } else {

                        $avatar_url = get_avatar_url( $email, array( 'size' => 50 ) );
                        $content .= '<img alt="" src="' . $avatar_url . '" class="avatar avatar-50 photo" height="50" width="50" />';

                    }

                    $content .= '<div class="fomo-text ' . $fomo_image . '">';
                    $content .= '<span class="fomo-name"><strong>' . $firstname . '</strong> donated </span>';
                    $content .= '<span class="fomo-product">'. number_format( $total ,2) . '</span>';
                    $content .=  ' to <span class="fomo-product-title">' . $formtitle . '</span>';
                    $content .= '<span class="fomo-subtext">Thank you!</span>';
                    $content .= '</div>';

                endwhile;
                wp_reset_postdata(); // end of Query 1

            endif;

            wp_reset_query();

            return apply_filters( 'hwp_fomo_content', $content, $post_id, $box_id );
        }

        /**
         *  Show MemberPress Information
         *
         *  This query outputs a random completed transaction
         *  with the name of the customer and the amount
         */
        public function memberpress_sale( $int, $box_id ) {

            $fomo_image = get_post_meta( $box_id, 'fomo_image', 1 );

            $total = $this->count_posts( 'memberpressproduct' );

            if( $total < 10 )
                $int = rand( 0, intval( $total ) - 1 );

            // If past 10 posts, get a random one
            if( $int > 10 )
                $int = rand( 1, 10 );

            global $wpdb;
            $mepr_db = new MeprDb();

            $q = $wpdb->prepare("
                SELECT id
                  FROM {$mepr_db->transactions}
                 WHERE status='complete'
                 ORDER BY created_at DESC
                 LIMIT 1
                 OFFSET %d
                ",
                $int
            );

            $transaction_id = $wpdb->get_var($q);

            $content = '';

            if( !empty( $transaction_id ) ) {

                $transaction = new MeprTransaction($transaction_id);
                $user = $transaction->user();
                $membership = $transaction->product();
                $last_name = $user->last_name;
                $last_initial = empty($last_name) ? '' : strtoupper($last_name[0]) . '.';

                // If using product image setting, show that instead of avatar
                if( $fomo_image === 'product') {

                    $product_image = get_the_post_thumbnail_url( $membership->ID, 'thumbnail' );
                    $content .= '<div class="hwp-product-image" style="background-image: url(' . $product_image . ');"></div>';

                } else {

                    $email = $user->user_email;

                    $avatar_url = get_avatar_url( $email, array( 'size' => 50 ) );
                    $content .= '<img alt="" src="' . $avatar_url . '" class="avatar avatar-50 photo" height="50" width="50" />';

                }

                $content .= '<div class="fomo-text ' . $fomo_image . '">';
                $content .= '<span class="fomo-name"><strong>' . $user->first_name . ' ' . $last_initial . '</strong> recently purchased </span>';
                $content .=  '<span class="fomo-product-title"><a href="' . $membership->url() . '">' . $membership->post_title . '</a></span>';
                $content .= '</div>';

            }

            return apply_filters( 'hwp_fomo_content', $content, $transaction_id, $box_id );
        }

        /**
         *  Get Paid Memberships Pro Sale
         *
         */
        public function pmp_sale( $int, $box_id ) {
            
            //$fomo_image = get_post_m`eta( $box_id, 'fomo_image', 1 );

            global $wpdb, $pmpro_pages;

            // Make sure we have at least 10 orders, if so cache request
            $trans = get_transient( 'hwp_pmp_orders' );

            if( $trans === false ) {

                $countQuery = "SELECT membership_id FROM $wpdb->pmpro_membership_orders ORDER BY timestamp DESC LIMIT 10";
                $orders = $wpdb->get_results($countQuery);
                $total = count( $orders );

                if( $total < 10 ) {
                    $int = rand( 0, intval( $total ) - 1 );
                } else {
                    set_transient( 'hwp_pmp_orders', 10 );
                }

            }

            // If past 10 posts, get a random one
            if( $int > 10 )
                $int = rand( 1, 10 );

            //get order from PMPro
            $sqlQuery = "SELECT user_id, membership_id FROM $wpdb->pmpro_membership_orders ORDER BY timestamp DESC LIMIT 1 OFFSET " . $int;
            $last_pmpro_order = $wpdb->get_results($sqlQuery);

            if ( !empty( $last_pmpro_order ) ) {

                // display/hide avatar image per user.
                $user_info = get_userdata( $last_pmpro_order[0]->user_id );
                $avatar_url = get_avatar_url( $user_info->user_email, array( 'size' => 50 ) );

                $avatar = '<img alt="" src="' . $avatar_url . '" class="avatar avatar-50 photo" height="50" width="50" />';
                $content .= apply_filters( 'pmprohwp_member_avatar', $avatar, $last_pmpro_order[0]->user_id );

                $membership_level = pmpro_getMembershipLevelForUser($last_pmpro_order[0]->user_id);
                $levels_page_title = get_the_title( $pmpro_pages['levels'] );
                $checkout_url = pmpro_url( 'checkout', '?level=' . $last_pmpro_order[0]->membership_id );

                $content .= '<div class="fomo-text">';

                //generate the text for hollerbox
                $text = sprintf( __( '<span class="fomo-name"><strong>%s</strong> recently signed up for the</span> %s. %s', 'pmpro-hollerbox' ), ucfirst( $user_info->display_name ), "<span class=\"fomo-product-title\"><a class=\"pmpro_holler_box_link\" href=\"" . $checkout_url . " \">" . $membership_level->name . "</a> membership</span>", "<a class='pmpro_hollerbox_link' href=\"" . pmpro_url("levels") . " \">" . $levels_page_title . "</a>" );
                
                $content .= $text;

                $content .= '</div>';

            }

            return apply_filters( 'hwp_fomo_content', $content, $last_pmpro_order[0]->membership_id, $box_id );
        }

        /**
         *  Determine if at least 10 posts exist, otherwise we can't use offset properly
         */
        public function count_posts( $type ) {

            $num = wp_count_posts( $type );

            return $num->publish;

        }

    }

    $holler_sales_functions = new Holler_Sales_Functions();
    $holler_sales_functions->instance();

} // end class_exists check